/* Electronic Voting Project
 * Mark A. Herschberg (hershey@mit.edu)
 */

package evote.crypto;

import java.math.BigInteger;
import java.security.*;
import java.util.Random;

import evote.util.constants.*;
import evote.util.genrand.*;

/**
  Creates a Message Authenitcation Code as the has of the message.
  */
public class ShaMACIntegrity implements IntegrityClass {
  ShaCommitmentKeys keys;
  Constants cons = new Constants();

  /**
    Creates an object to performs the MAC (including keys for SHA), using a
    byte array as a seed.
   */
  public ShaMACIntegrity(byte[] seed) {
    SecureRandom SRNG = new SecureRandom(seed);
    byte randomBytes[] = new byte[cons.numbytes];
    SRNG.nextBytes(randomBytes);
    keys.k1 = new BigInteger(cons.hashKeyLength, randomBytes);
    SRNG.nextBytes(randomBytes);
    keys.k2 = new BigInteger(cons.hashKeyLength, randomBytes);
  }
 
  /**
    Creates an object to performs the MAC (including keys for SHA), using a
    a GenRand object as a seed.
    */
  public ShaMACIntegrity(GenRand RNG) {
    keys= new ShaCommitmentKeys(new BigInteger(1, RNG.nextBytes(cons.hashKeyLength)), new BigInteger(1, RNG.nextBytes(cons.hashKeyLength)));
  }
  
  /**
    Creates an object to performs the MAC with the keys given.
    */
  public ShaMACIntegrity(ShaCommitmentKeys givenKeys) {
    keys = givenKeys;
  }
  
  /**
    This method simply calls another which performs the actual commitment.
   */
  public byte[] makeChecksum(byte[] b) {
    ShaCommitment ShaMac = new ShaCommitment(keys);
    return ShaMac.commit(b);
  }
  
  /**
    Returns the keys used in the MAC.
    */
  public IntegrityKeys getKeys() {
    return(keys);
  }

  /**
    Verify's a MAC (byte array) by checking it against the plaintext and keys
    in the object.
    */
  public boolean verify(byte[] msg, byte[] alledged) {
    ShaCommitment ShaMac = new ShaCommitment(keys);
    if (0 == (new BigInteger(alledged)).compareTo(new BigInteger(ShaMac.commit(msg)))) {
      return true;
    }
    else {
      return false;
    }
  }
  
}

