/*
 * This class performs the loading of the Election object
 */

package evote.election;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.math.*;

import evote.lang.*;
import evote.crypto.*;

public class ElectionLoader implements ActionListener {

    private Button loadElectionButton, okButton;
    private TextField electionHashField;
    private boolean electionLoaded= false;
    private Election myElection;

    private Frame thisFrame;

    public ElectionLoader() {
	init();
    }

    private void init() {
	thisFrame= new Frame("Election Loader");
    
	thisFrame.add("North",new Label("Please load Election object to continue", Label.CENTER));
    
	Panel centerPanel= new Panel();
	loadElectionButton= new Button("Load Election");
	loadElectionButton.addActionListener(this);
	okButton= new Button("go");
	okButton.addActionListener(this);
	okButton.setEnabled(false);
	centerPanel.add(loadElectionButton);
	centerPanel.add(okButton);
	thisFrame.add("Center", centerPanel);

	electionHashField= new TextField();
	electionHashField.setEditable(false);
	thisFrame.add("South", electionHashField);
    
	thisFrame.pack();
	thisFrame.setSize(500,200);
	thisFrame.show();
    }

    public ElectionLoader(String filename) {
	init(filename);
    }

    public void init(String filename) {
	try {
	    FileInputStream fis= new FileInputStream(filename);
	    ObjectInputStream ois= new ObjectInputStream(fis);
      
	    myElection= (Election)ois.readObject();

	    electionLoaded= true;
	    ois.close();
	    fis.close();
	} catch(Exception exc) {
	    exc.printStackTrace();
	}
    }

    public ElectionLoader(String[] argv) {
	if (argv.length == 0) {
	    init();
	    return;
	}
	
	init(argv[0]);
    }

    public synchronized Election getElection() {
	while(!electionLoaded) {
	    try {
		wait();
	    }
	    catch(InterruptedException exc) {
	    }
	}

	System.out.println("got an election object, about to return");

	this.dispose();
	return(myElection);
    }

    public void dispose() {
	if (thisFrame != null) {
	    thisFrame.dispose();
	    thisFrame= null;
	}
    }

    public void actionPerformed(ActionEvent ae) {
	Object source = ae.getSource();
    
	if (source == loadElectionButton) {
	    try {
		FileDialog fd= new FileDialog(thisFrame);
		fd.show();
		FileInputStream fis= new FileInputStream(new File(fd.getDirectory(), fd.getFile()));
		ObjectInputStream ois= new ObjectInputStream(fis);
	
		myElection=(Election) ois.readObject();
	
		if (myElection == null)
		    System.out.println("null at reading");

		showHash();
		okButton.setEnabled(true);
	    }
	    catch(Exception exc) {
		electionLoaded= false;
		okButton.setEnabled(false);
		System.out.println("exception : " + exc.toString());
		exc.printStackTrace();
	    }
	}

	if (source == okButton) {
	    if (myElection != null) {
		electionLoaded= true;
		synchronized(this) {
		    notifyAll();
		}
	    }
	}
      
    }

    public void showHash() {
	if (myElection == null)
	    return;

	Election copy= (Election)myElection.clone();
 
	try {
	    // take out the private keys for everything, and the userlists
	    copy.makeHashable();
      
	    ShaCommitmentKeys sha_keys= new ShaCommitmentKeys(new BigInteger("0"), new BigInteger("0"));
	    ShaCommitment sha= new ShaCommitment(sha_keys);
      
	    byte[] hash= sha.commit(copy.serialize());
      
	    // we have 5 ints
	    int[] hash_ints= new int[5];
	    for (int i=0; i<5; i++) {
		hash_ints[i]= 
		    ((hash[4*i] & 0xff) << 24) +
		    ((hash[4*i + 1] & 0xff) << 16) +
		    ((hash[4*i + 2] & 0xff) << 8) +
		    ((hash[4*i + 3] & 0xff));
	    }

	    String hash_string= "";
	    for (int i=0; i<5; i++)
		hash_string+= Integer.toHexString(hash_ints[i]);

	    electionHashField.setText("Election ID: " + hash_string);

	} catch(Exception exc) {
	    electionHashField.setText("Couldn't get Election ID : " + exc.toString());
	    exc.printStackTrace();
	}
    }

}    
