/*
 * QuestionList and UserList Setup
 * This panel allows a user to
 * choose the various
 */

package evote.election.builder;

import evote.lang.*;
import evote.election.*;
import evote.question.*;
import evote.question.builder.*;
import evote.identity.*;
import evote.registrar.*;

import java.awt.*;
import java.awt.event.*;


public class QuestionUserSetupPanel extends Panel implements ActionListener, ItemListener, QuestionListReceiver, UserListReceiver {
  
  private Election myElection;
  
  /* GUI */
  private List userlistList, questionlistList, selectionList;
  private Button addUserListButton, delUserListButton, addQuestionListButton, delQuestionListButton;
  private TextField selectionTitle;
  
  private boolean selectionMode;

  private boolean forEditing= false;
  private QuestionListBuilder myQLB;
  private Registrar myRegistrar;

  private static final boolean USERLIST_SELECTION= true;
  private static final boolean QUESTIONLIST_SELECTION= false;

  public QuestionUserSetupPanel(Election elec) {
    buildGUI();
    init(elec);
  }

  private void buildGUI() {
    userlistList= new List();
    questionlistList= new List();

    userlistList.addItemListener(this);
    userlistList.addActionListener(this);
    questionlistList.addActionListener(this);
    questionlistList.addItemListener(this);
    
    addUserListButton= new Button("Add");
    delUserListButton= new Button("Delete");
    addQuestionListButton= new Button("Add");
    delQuestionListButton= new Button("Delete");
    
    addUserListButton.addActionListener(this);
    delUserListButton.addActionListener(this);
    addQuestionListButton.addActionListener(this);
    delQuestionListButton.addActionListener(this);
    
    selectionTitle= new TextField();
    selectionTitle.setEditable(false);

    selectionList= new List();
    selectionList.setMultipleMode(true);
    selectionList.addItemListener(this);

    Panel bottomPanel= new Panel();
    bottomPanel.setLayout(new BorderLayout());
    bottomPanel.add("North", selectionTitle);
    bottomPanel.add("Center", selectionList);
    
    this.setLayout(new GridLayout(2,1));
    Panel topPanel= new Panel();
    topPanel.setLayout(new GridLayout(1,2));
    
    Panel topLeftPanel= new Panel();
    topLeftPanel.setLayout(new BorderLayout());
    topLeftPanel.add("Center", userlistList);
    topLeftPanel.add("North", new Label("User Lists",Label.CENTER));

    Panel topLeftBottom= new Panel();
    topLeftBottom.add(addUserListButton);
    topLeftBottom.add(delUserListButton);
    delUserListButton.setEnabled(false);

    topLeftPanel.add("South",topLeftBottom);
    
    topPanel.add(topLeftPanel);
    
    Panel topRightPanel= new Panel();
    topRightPanel.setLayout(new BorderLayout());
    topRightPanel.add("Center",questionlistList);
    topRightPanel.add("North", new Label("Question Lists", Label.CENTER));
    
    Panel topRightBottom= new Panel();
    topRightBottom.add(addQuestionListButton);
    topRightBottom.add(delQuestionListButton);
    delQuestionListButton.setEnabled(false);
    
    topRightPanel.add("South", topRightBottom);
    
    topPanel.add(topRightPanel);
    this.add(topPanel);
    this.add(bottomPanel);
  }
    
  private void updateQuestionList() {
    QuestionList[] list= myElection.getQuestionLists();
    
    clearSelectionPanel();
    questionlistList.removeAll();

    if (list == null)
      return;

    for (int i=0; i<list.length; i++) {
      String toAdd= "QuestionList #"+i+": ";
      if (list[i].getName()!=null)
	toAdd+= list[i].getName();
      questionlistList.add(toAdd,i);
    }
  }

  private void updateUserList() {
    UserList[] list= myElection.getUserLists();
    
    clearSelectionPanel();
    userlistList.removeAll();

    if (list == null)
      return;

    for (int i=0; i<list.length; i++) {
      String toAdd= "UserList #"+i+": ";
      if (list[i].getName()!=null)
	toAdd+= list[i].getName();
      userlistList.add(toAdd,i);
    }
  }

  private void clearSelectionPanel() {
    selectionTitle.setText("");
    selectionList.removeAll();
  }
  
  private void init(Election elec) {
    myElection= elec;
    updateQuestionList();
    updateUserList();
  }

  public void actionPerformed(ActionEvent ae) {
    Object source= ae.getSource();

    if (source == addQuestionListButton) {
      delQuestionListButton.setEnabled(false);
      delUserListButton.setEnabled(false);

      if (myQLB == null) {
	forEditing= false;
	addUserListButton.setEnabled(false);
	addQuestionListButton.setLabel("CANCEL");
	myQLB= new QuestionListBuilder(this);
      } else {
	myQLB.dispose();
	myQLB= null;
	addUserListButton.setEnabled(true);
	addQuestionListButton.setLabel("Add");
      }
    }
    
    if (source == addUserListButton) {
      delQuestionListButton.setEnabled(false);
      delUserListButton.setEnabled(false);

      if (myRegistrar == null) {
	forEditing= false;
	addQuestionListButton.setEnabled(false);
	addUserListButton.setLabel("CANCEL");
	myRegistrar= new Registrar(this);
      } else {
	myRegistrar.dispose();
	myRegistrar= null;
	addQuestionListButton.setEnabled(true);
	addUserListButton.setLabel("Add");
      }
    }

    if (source == delUserListButton) {
      myElection.delUserList(userlistList.getSelectedIndex());
      updateUserList();
    }

    if (source == delQuestionListButton) {
      myElection.delQuestionList(questionlistList.getSelectedIndex());
      updateQuestionList();
    }
	
    if (source == userlistList) {
      /* If there is already a registrar open,
       * then don't open another one
       */
      if ((myRegistrar != null) || (myQLB != null))
	return;

      delQuestionListButton.setEnabled(false);
      delUserListButton.setEnabled(false);

      forEditing= true;
      addQuestionListButton.setEnabled(false);
      addUserListButton.setEnabled(false);
      myRegistrar= new Registrar(this);
      myRegistrar.init(myElection.getUserList(userlistList.getSelectedIndex()));
    }

    if (source == questionlistList) {
      /* If there is already a question list builder open,
       * don't open it
       */
      if ((myQLB != null) || (myRegistrar != null))
	return;

      delQuestionListButton.setEnabled(false);
      delUserListButton.setEnabled(false);
      
      forEditing= true;
      addUserListButton.setEnabled(false);
      addQuestionListButton.setEnabled(false);
      myQLB= new QuestionListBuilder(this);
      myQLB.init(myElection.getQuestionList(questionlistList.getSelectedIndex()));
    }

  }

  public void itemStateChanged(ItemEvent ie) {
    Object source= ie.getSource();

    if (source == userlistList) {
      int sel= questionlistList.getSelectedIndex();
      if (sel>-1)
	questionlistList.deselect(sel);
     
      UserList selectedUserList= myElection.getUserList(userlistList.getSelectedIndex());
      showSelection(selectedUserList);
      
      /* State of Buttons */
      delUserListButton.setEnabled(true);
      delQuestionListButton.setEnabled(false);

      return;
    }

    if (source == questionlistList) {
      int sel= userlistList.getSelectedIndex();
      if (sel>-1)
	userlistList.deselect(sel);

      QuestionList selectedQuestionList= myElection.getQuestionList(questionlistList.getSelectedIndex());
      showSelection(selectedQuestionList);

      /* State of Buttons */
      delQuestionListButton.setEnabled(true);
      delUserListButton.setEnabled(false);

      return;
    }

    if (source == selectionList) {
      for (int i=0; i<selectionList.getItemCount(); i++) {
	if (selectionMode == USERLIST_SELECTION) 
	  myElection.setMatching(i, questionlistList.getSelectedIndex(), selectionList.isIndexSelected(i));
	else
	  myElection.setMatching(userlistList.getSelectedIndex(), i, selectionList.isIndexSelected(i));
      }
    }
	  
  }

  public void showSelection(QuestionList ql) {

    selectionMode= USERLIST_SELECTION;

    selectionList.removeAll();
    
    for (int i=0; i<userlistList.getItemCount(); i++) {
      selectionList.add(userlistList.getItem(i),i);
      if (myElection.getMatching(i, questionlistList.getSelectedIndex()))
	selectionList.select(i);
    }
    
    selectionTitle.setText("Userlists answering the " + ql.getName() + " Question List");
  }    

  public void showSelection(UserList ul) {

    selectionMode= QUESTIONLIST_SELECTION;

    selectionList.removeAll();
    
    for (int i=0; i<questionlistList.getItemCount(); i++) {
      selectionList.add(questionlistList.getItem(i),i);
      if (myElection.getMatching(userlistList.getSelectedIndex(), i))
	selectionList.select(i);
    }
    
    selectionTitle.setText("Question Lists answered by the " + ul.getName() + " User List");
  }    

  public void receiveUserList(UserList ul) {
    if (!forEditing)
      myElection.addUserList(ul);

    updateUserList();
    myRegistrar= null;
    addQuestionListButton.setEnabled(true);
    addUserListButton.setEnabled(true);
    addUserListButton.setLabel("Add");

    int sel= questionlistList.getSelectedIndex();
    if (sel>-1)
      questionlistList.deselect(sel);
  }

  public void receiveQuestionList(QuestionList ql) {
    if (!forEditing)
      myElection.addQuestionList(ql);

    updateQuestionList();
    myQLB= null;
    addQuestionListButton.setLabel("Add");
    addQuestionListButton.setEnabled(true);
    addUserListButton.setEnabled(true);

    int sel= userlistList.getSelectedIndex();
    if (sel>-1)
      userlistList.deselect(sel);
  }

}
