/*
 * UserList class
 * defines a user list
 */

package evote.identity;

import java.io.*;
import java.util.*;

public class UserList implements Serializable {
  
  private Vector users;
  private String name;

  public UserList() {
    users= new Vector();
  }
  
  /* Add a user to the userlist
   * This won't allow a user with an existing username to be added
   */
  public synchronized void addUser(User u) throws UserNameTakenException {
    User found= findUser(u.getUserName());
    if (found == null)
      users.addElement(u);
    else {
      if (found == u)
	return;
      else
	throw(new UserNameTakenException("A User with that username already exists"));
    }
  }

  /* Set Name
   */
  public void setName(String n){
    this.name= n;
  }

  /* Get Name
   */
  public String getName() {
    return(name);
  }

  /* Find if 
   * a user is there 
   */
  public boolean hasUser(User u) {
    if (findUser(u.getUserName()) == u) {
      return(true);
    } else {
      return (false);
    }
  }

  /* Delete a user from the user list
   */
  public synchronized void deleteUser(User u) {
    if (u != null)
      users.removeElement(u);
  }

  /* This finds a given user in the user list,
   * given a username
   */
  public User findUser(String username) {
    for (int i=0; i< users.size(); i++) {
      if (((User)users.elementAt(i)).getUserName().equals(username))
	return((User)users.elementAt(i));
    }
    return(null);
  }

  /*
   * This appends one userlist to another
   * duplicate username
   */
  public void append(UserList other_list) throws UserNameTakenException{
    for (int i=0; i< other_list.users.size(); i++) {
      addUser((User)other_list.users.elementAt(i));
    }
  }

  /*
   * Clears all the voting info
   */
  public void clear() {
    for (int i=0; i< users.size(); i++)
      ((User)users.elementAt(i)).clear();
  }

  /*
   * This lists users in the user list
   */
  public User[] getUsers() {
    User[] return_list= new User[users.size()];
    
    for (int i=0; i<return_list.length; i++) {
      return_list[i]= (User)users.elementAt(i);
    }

    return(return_list);
  }

  /* This clones the user list
   */
  public Object clone() {
    UserList return_ul= new UserList();
    
    return_ul.users= (Vector)this.users.clone();
    
    return(return_ul);
  }
}
