/*
 * This live object checks the time
 * on a TimeControllable object, and calls the right methods
 */

package evote.lang;

import java.util.*;

public class TimeController implements Runnable {
  
  private TimeControllable myObject;
  private Date startTime, endTime;
  private Thread myThread;
  private TimeControllerGUI myGUI;

  private boolean GUI_disabled= false;
  
  public TimeController(TimeControllable tc, Date dStart, Date dEnd) {
    myObject= tc;
    startTime= dStart;
    endTime= dEnd;

    myThread= new Thread(this);
    myThread.start();
  }

  public void disableGUI() {
    GUI_disabled=true;
  }  

  public void run() {

    if (!GUI_disabled)
      myGUI= new TimeControllerGUI(startTime, endTime);
    
    /* We wait for the start time, then
     * break out of the loop
     */
    while(true) {
      Date now= new Date();
      if (now.after(startTime))
	break;
      
      if (myGUI != null)
	myGUI.update(now);

      try {
	Thread.sleep(60000);
      } catch(InterruptedException exc) {
      }
      
    }

    if (myGUI != null)
      myGUI.started();
    // we need to start
    myObject.timeStart();

    /* We wait for the end time,
     * then break out of the loop
     */
    while(true) {
      try {
	Thread.sleep(60000);
      }
      catch(InterruptedException exc) {
      }

      Date now= new Date();
      if (now.after(endTime))
	break;
      
      if (myGUI != null)
	myGUI.update(now);
    }

    System.out.println("System stopping -- TIME CONTROLLER");
    myObject.timeStop();

    if (myGUI != null)
      myGUI.stopped();
  }
}

