/* Electronic Voting Project
 * This is the listener to blacklist events
 * Ben Adida (ben@mit.edu)
 */

package evote.network.server;

import java.util.EventListener;
import java.util.EventObject;
import evote.network.server.BlackListEvent;

/**
 * Classes that implement this interface can receive
 * Black-list related information from the Server.
 * Usually, GUI's will implement this interface.
 */
public interface BlackListListener extends EventListener {
  /**
   * Add an element to the black list
   *
   * @argument ble The Black list event that describes what happened
   */
  public void addBlackListedElement(BlackListEvent ble);

  /**
   * Remove an element from the black list
   *
   * @argument ble The Black list event that describes what happened
   */
  public void removeBlackListedElement(BlackListEvent ble);

  /**
   * Clear the black list
   *
   * @argument ble The Black list event that describes what happened
   */
  public void clearBlackListedElements(BlackListEvent ble);
}
