/* Electronic Voting Project
 * Connection Handler interface
 * written by Ben Adida (ben@mit.edu)
 */

package evote.network.server;

import java.io.*;
import java.net.*;

/**
 * A connection handler
 *
 * This interface defines the properties of a server-side connection handler.
 * The <pre>Server</pre> uses a <pre>ConnectionHandler</pre> to handle the
 * incoming connections it receives.
 */
public interface ConnectionHandler {

  /**
   * This method is called when a connection needs to be handled
   * Often, this method will be called simultaneously by different 
   * threads: serialization provisions must be taken where 
   * necessary
   *
   * @argument c The connection that needs to be handled
   */
  public void handleConnection(Connection c) throws InterruptedIOException;

  /**
   * This method is called when an error happens with the Connection.
   * Again, this may be called by multiple threads simultaneously.
   *
   * @argument e the Connection Error.
   */
  public void handleError(ConnectionError e);

  /**
   * This method is called before the server shuts down
   * if stuff needs to be done
   *
   */
  public void serverShutdown();
}
