/* Electronic Voting Package
 * The Logger
 * Ben Adida (ben@mit.edu)
 */

package evote.network.server;

import evote.network.server.*;
import java.io.*;

/** 
 * A logger
 *
 * This class exposes an interface to logging events to disk.
 *
 * @see evote.network.server.LogEntry
 */
public class Log {
  private FileOutputStream fos;
  private ObjectOutputStream oos;
  
  /**
   * This constructs a logger
   *
   * @argument logname The name of the file that the logger logs to.
   * @return The requested Log
   * @exception evote.network.server.LogException
   *   If any sort of exception happens when a Log is created,
   *   this exception is thrown
   */
  public Log(String logname) throws LogException {
    try {
      fos= new FileOutputStream(logname);
      oos= new ObjectOutputStream(fos);
    }
    catch (IOException ioe) {
      throw(new LogException("Logger could not start log file: " + ioe.toString()));
    }
  }

  /**
   * This adds a log entry to the log
   *
   * @argument le The LogEntry to add.
   * @exception evote.network.server.LogException
   *    If the log entry cannot be added to the log, a log exception
   *    happens
   */
  public synchronized void addLogEntry(LogEntry le) throws LogException {
    try {
      oos.writeObject(le);
    }
    catch(IOException e) {
      throw(new LogException("couldn't log entry: " + e));
    }
  }

  /** 
   * Closes the log
   */
  public void close() {
    try {
      oos.close();
      fos.close();
    }
    catch(Exception e) {
    }
  }
}
    
