/* Electronic Voting Project
 * Log Entry
 * Ben Adida
 */

package evote.network.server;

import java.util.Date;
import java.io.Serializable;

/** 
 * A Log entry
 * 
 * This is a simple wrapper class that abstracts the concept
 * of a log entry.
 *
 * @see evote.network.server.Log
 */
public class LogEntry implements Serializable {

  /* Types of entries */
  public static final int NORMAL_LOG= 0;
  public static final int WARNING_LOG= 1;
  public static final int SERIOUS_LOG= 2;
  public static final int FATAL_LOG= 3;

  private Date when;
  private int status;
  private String description;

  /* A LOG ENTRY IS IMMUTABLE */

  /**
   * Create a LogEntry
   * 
   * This simply creates a new log entry.
   * The date of the log entry is created upon creation
   * of this object.
   *
   * @argument stat The status of the log entry, its gravity.
   * @argument desc The description of the log entry.
   * @return The created log entry.
   */
  public LogEntry(int stat, String desc) {
    this.when= new Date();
    this.status= stat;
    this.description= desc;
  }

  /**
   * This returns the status of the Log Entry
   *
   * @return The integer of the status.
   */
  public int getStatus() {
    return(this.status);
  }

  /** 
   * This returns the date of the log entry
   *
   * @return The date of the log entry
   */
  public Date getDate() {
    Date d= new Date(when.toString());
    return(d);
  }

  /** 
   * This returns the string description of the log
   *
   * @return the string of the description 
   */
  public String getDescription() {
    return(this.description);
  }
}
