/*
 * This program parses the list from the Registrar and creates
 * userlists for the Evox system
 */

package evote.parser;

import evote.identity.*;
import java.util.*;
import java.io.*;

public class UserListParser {
  
  private static ObjectOutputStream seniorStream, juniorStream, sophomoreStream, freshmanStream;
  private static DataInputStream dis;

  public static void main(String argv[]) {
    if (argv.length != 1) {
      System.err.println("incorrect usage");
      System.exit(0);
    }

    System.out.println("file: "  + argv[0]);

    try {
      dis= new DataInputStream(new FileInputStream(argv[0]));
    } 
    catch (IOException exc) {
      System.err.println("Couldn't open input file");
      System.exit(0);
    }


    /* Create the user lists */
    UserList seniorList, juniorList, sophomoreList, freshmanList;
    seniorList= new UserList();
    seniorList.setName("Seniors");
    juniorList= new UserList();
    juniorList.setName("Juniors");
    sophomoreList= new UserList();
    sophomoreList.setName("Sophomores");
    freshmanList= new UserList();
    freshmanList.setName("Freshmen");

    /* Start the loop for reading */
    /* The format of the line is:
     * username \t first_name \t middle_initial \t last_name \t year
     */
    
    String line,uname,lname,fname,email, year;
    StringTokenizer tok;
    User currentUser;
    int year_num=0;

    while (true) {
      try {
	line= dis.readLine();
      }
      catch(Exception exc) {
	break;
      }

      if ((line == null)||(line==""))
	break;

      try {
	tok= new StringTokenizer(line);
	uname= tok.nextToken();
	fname= "";
	lname= "";
	email= uname + "@mit.edu";
	while (true) {
	  year= tok.nextToken();
	  try {
	    year_num= Integer.parseInt(year);
	  } catch(NumberFormatException exc) {
	    fname+= " " + year;
	    continue;
	  }
	  break;
	}
      }
      catch(NoSuchElementException exc) {
	System.err.println("parsing error for \n " + line + "\n");
	System.err.println("ignoring that user");
	continue;
      }
	
      currentUser= new User(uname,lname,fname,email);
      currentUser.setPassword(null, uname);

      try {
	System.out.print("" + year_num);
	if (year_num==1)
	  freshmanList.addUser(currentUser);
	if (year_num==2)
	  sophomoreList.addUser(currentUser);
	if (year_num==3)
	  juniorList.addUser(currentUser);
	if (year_num==4)
	  seniorList.addUser(currentUser);
      } catch(UserNameTakenException exc) {
	System.err.println("USER NAME TAKEN!!");
	System.exit(0);
      }
    
      System.out.print(".");
    }
    System.out.println("done");

    /* Write the userlists out */
    try {
      freshmanStream= new ObjectOutputStream(new FileOutputStream("freshmen.userlist"));
      sophomoreStream= new ObjectOutputStream(new FileOutputStream("sophomores.userlist"));
      juniorStream= new ObjectOutputStream(new FileOutputStream("juniors.userlist"));
      seniorStream= new ObjectOutputStream(new FileOutputStream("seniors.userlist"));
      
      freshmanStream.writeObject(freshmanList);
      sophomoreStream.writeObject(sophomoreList);
      juniorStream.writeObject(juniorList);
      seniorStream.writeObject(seniorList);

      freshmanStream.close();
      sophomoreStream.close();
      juniorStream.close();
      seniorStream.close();
    }
    catch(Exception exc) {
      System.err.println("Couldn't write out the userlist files");
    }

  }
}
      
