/* 
 * This is a Question List Builder
 * It is a GUI
 */

package evote.question.builder;

import evote.question.*;
import evote.lang.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class QuestionListBuilder extends Frame implements ActionListener {

  private static final String ALLOW_WRITE_INS= "Allow write-ins";
  private static final String DONT_ALLOW_WRITE_INS= "Don't allow write-ins";

  private QuestionList myQuestionList;
  private QuestionListReceiver myReceiver;
  private String directory, file;
  private int currentQuestion= -1;

  /* GUI Elements */
  private QuestionEditPanel[] questionPanels;
  private CardLayout myCardLayout;
  
  private MenuBar myMenuBar;
  private Menu questionListMenu;
  private MenuItem loadFileItem, saveFileItem, exitItem;

  private TextField numQuestionsField, numAnswersField, questionField, nameField;
  private TextArea questionArea;
  private Button createQuestionListButton, createQuestionButton, 
    nextQuestionButton, prevQuestionButton;
  private Panel mainPanel, numAnswersPanel;
  private TextField titleField;
  private java.awt.Choice allowsWriteInsChoice;

  private Button prevButton, nextButton, commitButton, editQuestionButton;
  private static final String NUM_ANSWERS= "*NUM ANSWERS*";
  private static final String QUESTION= "*QUESTION ";
  private static final String START= "*START*";
  
  public QuestionListBuilder(QuestionListReceiver qlr) {
    super("Question List Builder");

    myReceiver= qlr;
    Panel startPanel = new Panel();
    
    /* The start panel has the question list
     * name and the number of questions defined
     */
    startPanel.setLayout(new GridLayout(3,1));
    Panel startTopPanel, startBottomPanel, startButtonPanel;
    startTopPanel= new Panel(new FlowLayout());
    startTopPanel.add(new Label("Number of Questions:"));
    numQuestionsField= new TextField(5);
    startTopPanel.add(numQuestionsField);
    
    startBottomPanel= new Panel(new FlowLayout());
    nameField= new TextField(20);
    startBottomPanel.add(new Label("Name:"));
    startBottomPanel.add(nameField);

    createQuestionListButton= new Button("Go");
    createQuestionListButton.addActionListener(this);
    startButtonPanel= new Panel();
    startButtonPanel.add(createQuestionListButton);

    startPanel.add(startTopPanel);
    startPanel.add(startBottomPanel);
    startPanel.add(startButtonPanel);

    createGUI();
    mainPanel.add(START,startPanel);

    myCardLayout.show(mainPanel,START);
  }
  
  public QuestionListBuilder(QuestionList ql, QuestionListReceiver qlr) {
    super("Question List Builder");

    createGUI();
    init(ql);
  }

  private void createGUI() {
    titleField= new TextField();
    titleField.setEditable(false);

    titleField.setText("Create a New Question List");

    /* Do the bottom panel */
    Panel bottomPanel= new Panel();

    prevButton= new Button("< Previous");
    prevButton.setEnabled(false);
    prevButton.addActionListener(this);
    
    nextButton= new Button("Next >");
    nextButton.setEnabled(false);
    nextButton.addActionListener(this);
    
    commitButton= new Button("Commit");
    commitButton.setEnabled(false);
    commitButton.addActionListener(this);
    
    editQuestionButton= new Button("Edit Question");
    editQuestionButton.setEnabled(false);
    editQuestionButton.addActionListener(this);
    
    bottomPanel.setLayout(new BorderLayout());
    Panel bottomLeftPanel= new Panel();
    bottomLeftPanel.add(prevButton);
    bottomLeftPanel.add(commitButton);
    bottomLeftPanel.add(nextButton);
    
    Panel bottomRightPanel= new Panel();
    bottomRightPanel.add(editQuestionButton);

    bottomPanel.add("Center",bottomLeftPanel);
    bottomPanel.add("East",bottomRightPanel);

    mainPanel= new Panel();
    myCardLayout= new CardLayout();
    mainPanel.setLayout(myCardLayout);
    
    this.add("South",bottomPanel);
    this.add("Center",mainPanel);
    this.add("North",titleField);
    
    /* Num Answers Panel */
    numAnswersPanel= new Panel(new GridLayout(4,1));

    Panel topNumAnswersPanel, bottomNumAnswersPanel, buttonNumAnswersPanel, writeInsPanel;
    topNumAnswersPanel= new Panel();
    bottomNumAnswersPanel= new Panel(new BorderLayout());
    buttonNumAnswersPanel= new Panel();
    writeInsPanel= new Panel();

    //questionField= new TextField(35);
    questionArea= new TextArea("", 5, 50, TextArea.SCROLLBARS_NONE);
    bottomNumAnswersPanel.add("North",new Label("Question:"));
    bottomNumAnswersPanel.add("Center",questionArea);
    topNumAnswersPanel.add(new Label("Number of Answers:"));
    numAnswersField= new TextField(5);
    topNumAnswersPanel.add(numAnswersField);
    createQuestionButton= new Button("Go");
    createQuestionButton.addActionListener(this);
    buttonNumAnswersPanel.add(createQuestionButton);

    allowsWriteInsChoice= new java.awt.Choice();
    allowsWriteInsChoice.add(ALLOW_WRITE_INS);
    allowsWriteInsChoice.add(DONT_ALLOW_WRITE_INS);
    writeInsPanel.add(allowsWriteInsChoice);

    numAnswersPanel.add(bottomNumAnswersPanel);
    numAnswersPanel.add(topNumAnswersPanel);
    numAnswersPanel.add(writeInsPanel);
    numAnswersPanel.add(buttonNumAnswersPanel);

    mainPanel.add(NUM_ANSWERS, numAnswersPanel);
    
    myMenuBar= new MenuBar();
    questionListMenu= new Menu("Question List");
    loadFileItem= new MenuItem("Load Question List");
    saveFileItem= new MenuItem("Save Question List");
    exitItem= new MenuItem("Exit");

    loadFileItem.addActionListener(this);
    saveFileItem.addActionListener(this);
    exitItem.addActionListener(this);
    
    questionListMenu.add(loadFileItem);
    questionListMenu.add(saveFileItem);
    questionListMenu.add(exitItem);
    
    myMenuBar.add(questionListMenu);
    
    this.setMenuBar(myMenuBar);

    this.pack();
    this.setSize(400,500);
    this.show();
  }
    
  private void init(int num_questions) {
    QuestionList ql= new QuestionList(num_questions);
    init(ql);
  }

  public void init(QuestionList ql) {
    myQuestionList= ql;

    questionPanels= new QuestionEditPanel[ql.getNumQuestions()];
    Question currentQuestion;    
    for (int i=0; i<ql.getNumQuestions(); i++) {
      currentQuestion= ql.getQuestion(i);
      if (currentQuestion != null) {
	questionPanels[i]= new QuestionEditPanel(this, currentQuestion, i, questionPanels.length);
	mainPanel.add(QUESTION+i,questionPanels[i]);
      }
    }
    showQuestion(0);
  }
  
  /* Show a particular question */
  private void showQuestion(int num) {
    int oldQuestion= currentQuestion;
    currentQuestion= num;

    titleField.setText("Question #"+(currentQuestion+1));

    /* What to do with previous question */
    if (oldQuestion > -1) {
      if (questionPanels[oldQuestion] != null)
	questionPanels[oldQuestion].commit();
    }

    if (questionPanels[num] == null) {
      /* There is no question there yet */
      editQuestionButton.setEnabled(false);
      commitButton.setEnabled(false);
      myCardLayout.show(mainPanel,NUM_ANSWERS);
    }
    else {
      /* There is a question there */
      commitButton.setLabel("Edit");
      commitButton.setEnabled(true);
      editQuestionButton.setEnabled(true);
      myCardLayout.show(mainPanel,QUESTION+currentQuestion);
    }

    /* Set proper enabling of buttons
     * for next, prev, commit
     */
    if (num == 0)
      prevButton.setEnabled(false);
    else
      prevButton.setEnabled(true);

    if (num == (myQuestionList.getNumQuestions() -1))
      nextButton.setEnabled(false);
    else
      nextButton.setEnabled(true);
  }

  /* next */
  public void next() {
    if ((currentQuestion != -1) && (currentQuestion<(questionPanels.length)))
      showQuestion(currentQuestion+1);
  }

  /* previous */
  public void previous() {
    if ((currentQuestion != -1) && (currentQuestion>0))
      showQuestion(currentQuestion-1);
  }

  public void actionPerformed(ActionEvent ae) {
    Object source= ae.getSource();

    if (source == nextButton) {
      showQuestion(currentQuestion+1);
      return;
    }

    if (source == prevButton) {
      showQuestion(currentQuestion-1);
      return;
    }

    if (source == commitButton) {
      if (questionPanels[currentQuestion].isEditable()) {
	questionPanels[currentQuestion].setEditable(false);
	questionPanels[currentQuestion].commit();
	commitButton.setLabel("Edit");
      } else {
	questionPanels[currentQuestion].setEditable(true);
	commitButton.setLabel("Commit");
      }
      return;
    }

    if (source == editQuestionButton) {
      questionPanels[currentQuestion]= null;
      myQuestionList.setQuestion(currentQuestion, null);
      showQuestion(currentQuestion);
      return;
    }

    if (source == createQuestionListButton) {
      /* Parse the number field */
      try {
	int num_questions= Integer.parseInt(numQuestionsField.getText());
	init(num_questions);
	myQuestionList.setName(nameField.getText());
      } catch(Exception exc) {
	System.out.println("Exception: " + exc.toString());
      }
      return;
    }

    if (source == createQuestionButton) {
      /* Parse the number field */
      try {
	int num_answers= Integer.parseInt(numAnswersField.getText());
	boolean allow_write_ins= false;
	if (allowsWriteInsChoice.getSelectedItem().equals(ALLOW_WRITE_INS))
	  allow_write_ins= true;
	Question newQuestion= new Question(questionArea.getText(), num_answers, allow_write_ins);
	myQuestionList.setQuestion(currentQuestion, newQuestion);
	questionPanels[currentQuestion]= new QuestionEditPanel(this,newQuestion,currentQuestion,questionPanels.length);
	mainPanel.add(QUESTION+currentQuestion,questionPanels[currentQuestion]);
	showQuestion(currentQuestion);
	questionArea.setText("");
	numAnswersField.setText("");
      }
      catch(Exception exc) {
	System.out.println("Oh oh: " + exc.toString());
	exc.printStackTrace();
      }
      return;
    }

    if (source == saveFileItem) {
      try {
	FileDialog fd= new FileDialog(this);
	if ((this.file != null) && (this.directory!= null)) {
	  fd.setDirectory(this.directory);
	  fd.setFile(this.file);
	}
	fd.show();
	this.directory= fd.getDirectory();
	this.file= fd.getFile();
	FileOutputStream fos= new FileOutputStream(new File(this.directory,this.file));
	ObjectOutputStream oos= new ObjectOutputStream(fos);
	oos.writeObject(myQuestionList);
	oos.close();
      } catch(Exception exc) {
	System.out.println("coudln't save file : " + exc.toString());
      }
      return;
    }

    /* Load menu item */
    if (source == loadFileItem) {
      try {
	FileDialog fd= new FileDialog(this);
	fd.show();
	this.directory= fd.getDirectory();
	this.file= fd.getFile();
	FileInputStream fis= new FileInputStream(new File(this.directory,this.file));
	ObjectInputStream ois= new ObjectInputStream(fis);
	QuestionList tempQuestionList= (QuestionList)ois.readObject();
	ois.close();
	
	init(tempQuestionList);
      } catch(Exception exc) {
	System.out.println("couldn't load file :  " + exc.toString());
      }
      return;
    }

    if (source == exitItem) {
      this.setVisible(false);
      if (myReceiver != null)
	myReceiver.receiveQuestionList(myQuestionList);
      this.dispose();
    }
  }

  public static void main(String argv[]) {
    new QuestionListBuilder(null);
  }
}
