/*
 * The New Registrar System
 */

package evote.registrar;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import evote.identity.*;
import evote.lang.*;

public class Registrar extends Frame implements WindowListener,ActionListener,ItemListener {

  private UserListReceiver myReceiver;

  /* components */
  private List userList;
  private Button addButton, delButton, editButton, editNameButton;
  private TextField nameField;
  private boolean nameEditable= false;

  /* Menus */
  private Menu registryMenu;
  private MenuItem loadFileItem, saveFileItem, quitItem;

  /* Stuff */
  private String directory,file;

  private UserList myUserList;

  public Registrar(UserListReceiver ulr) {

    myUserList= new UserList();
    myReceiver= ulr;

    /* MenuBar */
    MenuBar menubar= new MenuBar();

    /* Set up Menus */
    registryMenu= new Menu("Registry");
    loadFileItem= new MenuItem("Load Registry File");
    saveFileItem= new MenuItem("Save Registry File");
    quitItem= new MenuItem("Quit");

    loadFileItem.addActionListener(this);
    saveFileItem.addActionListener(this);
    quitItem.addActionListener(this);

    registryMenu.add(loadFileItem);
    registryMenu.add(saveFileItem);
    registryMenu.add(quitItem);

    menubar.add(registryMenu);

    this.setMenuBar(menubar);

    /* Components */
    nameField= new TextField();
    nameField.setEditable(false);
    editNameButton= new Button(" Edit ");
    editNameButton.addActionListener(this);

    userList= new List();

    userList.addActionListener(this);
    userList.addItemListener(this);

    addButton= new Button("Add");
    delButton= new Button("Delete");
    editButton= new Button("Edit");

    delButton.setEnabled(false);
    editButton.setEnabled(false);

    addButton.addActionListener(this);
    delButton.addActionListener(this);
    editButton.addActionListener(this);

    /* Layout */
    this.setLayout(new BorderLayout());

    Panel topPanel= new Panel();
    topPanel.setLayout(new BorderLayout());
    topPanel.add("Center",nameField);
    topPanel.add("East",editNameButton);
    
    this.add("North", topPanel);
    this.add("Center", userList);
    
    Panel bottom_panel= new Panel();
    bottom_panel.setLayout(new GridLayout(1,3));
    bottom_panel.add(addButton);
    bottom_panel.add(delButton);
    bottom_panel.add(editButton);
    
    this.add("South",bottom_panel);

    this.pack();
    this.setSize(300,300);
    this.show();
  }

  public void init(UserList ul) {
    myUserList= ul;

    nameField.setText(ul.getName());

    userList.removeAll();
    User[] users= ul.getUsers();
    if (users.length > 100)
      return;

    for (int i=0; i<users.length; i++) {
      userList.add(users[i].getUserName());
    }
    
    if (ul.getName() != null)
      nameField.setText(ul.getName());
  }

  public static void main(String argv[]) {
    Registrar reg= new Registrar(null);
  }

  public void add(String new_user) {
    userList.add(new_user);
  }

  public void delete(String user) {
    userList.remove(user);
  }

  public void actionPerformed(ActionEvent e) {
    Object source= e.getSource();

    /* Add Button is pressed */
    if (source == addButton) {
      AddUserWindow auw= new AddUserWindow(this,myUserList);
      return;
    }      

    /* Delete Button is pressed */
    if (source == delButton) {
      String userName= userList.getSelectedItem();
      myUserList.deleteUser(myUserList.findUser(userName));
      userList.remove(userName);
      return;
    }

    /* Edit Button is pressed */
    if (source == editButton) {
      String userName= userList.getSelectedItem();
      User thisUser= myUserList.findUser(userName);
      EditUserWindow euw= new EditUserWindow(thisUser, this);
      return;
    }

    /* Edit Name */
    if (source == editNameButton) {
      if (nameEditable) {
	nameEditable= false;
	nameField.setEditable(nameEditable);
	myUserList.setName(nameField.getText());
	editNameButton.setLabel(" Edit ");
      } else {
	nameEditable= true;
	nameField.setEditable(nameEditable);
	editNameButton.setLabel("Commit");
      }	
    }

    /* Save menu item */
    if (source == saveFileItem) {
      try {
	FileDialog fd= new FileDialog(this);
	if ((this.file != null) && (this.directory!= null)) {
	  fd.setDirectory(this.directory);
	  fd.setFile(this.file);
	}
	fd.show();
	this.directory= fd.getDirectory();
	this.file= fd.getFile();
	FileOutputStream fos= new FileOutputStream(new File(this.directory,this.file));
	ObjectOutputStream oos= new ObjectOutputStream(fos);
	oos.writeObject(myUserList);
	oos.close();
      } catch(Exception exc) {
	System.out.println("coudln't save file : " + exc.toString());
      }
      return;
    }

    /* Load menu item */
    if (source == loadFileItem) {
      try {
	FileDialog fd= new FileDialog(this);
	fd.show();
	this.directory= fd.getDirectory();
	this.file= fd.getFile();
	FileInputStream fis= new FileInputStream(new File(this.directory,this.file));
	ObjectInputStream ois= new ObjectInputStream(fis);
	UserList tempUserList= (UserList)ois.readObject();
	ois.close();
	
	init(tempUserList);
      } catch(Exception exc) {
	System.out.println("couldn't load file :  " + exc.toString());
      }
      return;
    }

    if (source == quitItem) {
      this.setVisible(false);
      if (myReceiver != null)
	myReceiver.receiveUserList(myUserList);
      this.dispose();
    }
  }
    
  public void itemStateChanged(ItemEvent e) {
    Object source= e.getSource();
    
    /* If something is SELECTED */
    if (e.getStateChange() == ItemEvent.SELECTED) {
      delButton.setEnabled(true);
      editButton.setEnabled(true);
    }
    
    /* If something is DESELECTED */
    if (e.getStateChange() == ItemEvent.DESELECTED) {
      delButton.setEnabled(false);
      editButton.setEnabled(false);
    }
  }

  public void windowActivated(WindowEvent e) {
  }

  public void windowClosed(WindowEvent e) {
  }
  
  public void windowClosing(WindowEvent e) {
  }

  public void windowDeactivated(WindowEvent e) {
  }

  public void windowDeiconified(WindowEvent e) {
  }

  public void windowIconified(WindowEvent e) {
  }

  public void windowOpened(WindowEvent e) {
  }


}
