
/*
 *  GenRandPanel.java
 *  ben@mit.edu
 */

package evote.util.genrand;

import evote.util.constants.*;
import evote.crypto.*;
import java.awt.*;
import java.applet.Applet;
import evote.lang.*;

public class GenRandPanel extends GenRand implements Runnable {
  /**
    The MessageDigest object md is used to hash the bytes generated by
    the user moving the cursor using the SHA algorithm.
    */
  //private MessageDigest md;
  /**
    The SecureRandom object sr is a link to Sun's pseudo random number
    generator (PRNG) and is used whenever the nextBytes(int numBytes)
    method is called.
    */
  private SecureRandom sr;

  private GetCursorPanelBytes gcb;
  private RandomDataRequestor return_rdr;

  public GenRandPanel() {
    super();
  }
  
  public Panel getPanel() {
    gcb = new GetCursorPanelBytes();
    return(gcb);
  }	    

  /**
    Call the seed() method to open a dialog that tracks the user's cursor
    movements until a certain number of bytes of data is generated.  Those
    bytes are then hashed using SHA (as provided in Sun's JDK).  Finally,
    that 160-bit hash is used to seed Sun's SecureRandom() pseudo random
    number generator (PRNG).
    */
  public void seed(RandomDataRequestor rdr) {
    // second step:  hash the bytes using SHA
    // 		try {
    // 		  md = MessageDigest.getInstance("SHA");
    // 		}
    // 		catch(Exception e) {
    // 			System.err.println("Error using SHA.");
    // 			System.exit(0);
    // 		}
    /**
      The byte array hashOut contains the 160-bit hash of the byte array
      generated by the user moving the cursor.
      */
		
    return_rdr= rdr;
    Thread seedThread= new Thread(this);
    seedThread.start();
  }

  public void run() {
    gcb.waitForData();
    SHADigest md= new SHADigest();
    byte hashOut[] = md.performHash(gcb.mouseData);

    // third step:  seed the SecureRandom PRNG
    sr = new SecureRandom(hashOut);
    return_rdr.randomDataGenerated();
  }    

  /**
		Using the SecureRandom() pseudo random number generator supplied by Sun,
		the specified number of random bytes are returned.  The seed() method
		must be called before the nextBytes() function is called.
	*/
  synchronized public byte[] nextBytes(int numBytes) {
    byte randomBytes[] = new byte[numBytes];
    
    sr.nextBytes(randomBytes);
    return randomBytes;
  }

  public static void main(String S[]) {
    GenRand gr = new GenRand();
    gr.seed();
    System.out.println("Twenty bytes returned from nextBytes(20):");
    byte result[] = gr.nextBytes(20);
		
    for (int i = 0; i < 20; i++)
      System.out.println(""+result[i]);
    System.out.println("");
  }

}

/**
	The GetCursorBytes dialog is used by GenRand to track the user's cursor
	movements and store that data in a byte array.  It should not be called
	by your program.  Use GenRand's seed() method instead.
*/
class GetCursorPanelBytes extends Panel {
  /**
    The numCursorBytes variable is the number of bytes to store using
    the coordinates of the cursor as the user moves it within the dialog.
    The default value is 200.  It must be a multiple of 2.
    */
  public int numCursorBytes = Constants.genrandNumCursorBytes;
  /**
    Array of bytes to store the coordinates of cursor.
    */
  public byte mouseData[] = new byte[numCursorBytes];
  /**
    Index for use with mouseData byte array.
    */
  private int i = 0;
  /**
    The offscreen canvas used in double buffering.
    */
  private Image OffscreenImage;
  /**
    The interface to the offscreen canvas.
    */
  private Graphics OffscreenGraphics;
  /**
    The Font object TextFont is used to determine the width and height of
    the text in the dialog.
    */
  private Font TextFont;
  /**
    The FontMetrics object TextFontMetrics contains metric information about
    the font used in the dialog.
    */
  private FontMetrics TextFontMetrics;
  /**
    The maximum height of a character can be found by adding the leading space,
    maximum ascent, and maximum descent of the dialog's font.
    */
  private int maxFontHeight;
  /**
    The width of the window is determined by using information about the metrics
    of the dialog's font.
    */
  private int winWidth;
  /**
    The height of the window is determined by using information about the metrics
    of the dialog's font.
    */
  private int winHeight;

  String line1 = new String("To securely send your vote, it is necessary to generate");
  String line2 = new String("random data.  The cursor's position will be continually");
  String line3 = new String("tracked until enough data is obtained.  To ensure");
  String line4 = new String("randomness, move the cursor quickly and erratically");
  String line5 = new String("within the boundary of this window.");
	
  DepressedBox border = new DepressedBox();

  ProgressBar bar = new ProgressBar();

  private boolean done= false;

  /**
    The constructor takes a frame as an argument because a frame is
    required to create a dialog.  The constructor sets the font and
    size of the dialog and the size and position of the progress bar.
    */
  public GetCursorPanelBytes() {
	  
    setLayout(new BorderLayout());
    this.setBackground(Color.white);

    //String font_name[] = TextFontMetrics.getFontList();
    TextFont = new Font("Dialog", Font.PLAIN, 12);
    TextFontMetrics = getFontMetrics(TextFont);
    maxFontHeight = TextFontMetrics.getLeading() + TextFontMetrics.getMaxAscent() + TextFontMetrics.getMaxDescent();
    setFont(TextFont);
		
    winWidth = TextFontMetrics.stringWidth(line1) + 50;

    // initialize the progress bar and border
    border.width	= bar.width		= winWidth - 20;
    border.height	= bar.height	= 15;
    border.x		= bar.x			= 10;
    border.y		= bar.y			= 60 + (maxFontHeight) * 5 + 10;

    winHeight = 60 + (maxFontHeight) * 5 + 10 + bar.height + 60;

    // resize(winWidth, winHeight);
  }

  protected void waitForData() {
    while (!done) {
      try {
	Thread.sleep(1000);
      } catch (InterruptedException exc) {
      }
    }
  }

  /**
    Whenever the mouse is moved, the x and y coordinates are cast to
    bytes and these bytes are stored in an array.  The screen is repainted
    so that the progress bar can be updated.  When enough bytes have been
    generated, the dialog is hidden with the hide() method.
    */
  public boolean mouseMove(Event evt, int x, int y) {
    if (i < numCursorBytes) {
      // store the least significant bytes of the x and y cursor coordinates
      mouseData[i++] = (byte) x;
      mouseData[i++] = (byte) y;
      repaint();
    }
    else {
      done= true;
      hide();
    }

    return true;
  }

  /**
    The update method is used in double buffering to prevent text and progress
    bar flicker.  An image with dimensions equal to that of the dialog is
    created and all drawing is done to that image.  When there is a break
    in the action, that image is swapped to the screen.
    */
  public synchronized void update(Graphics g) {
    if (OffscreenImage == null) {
      // make a new image the same size as the dialog
      OffscreenImage = createImage(winWidth,winHeight);
      // bind our interface to the Image
      OffscreenGraphics = OffscreenImage.getGraphics();
    }

    // do applet painting to our canvas
    paint(OffscreenGraphics);
    // paint the real graphics window
    g.drawImage(OffscreenImage,0,0,this);
  }

  /**
    When the screen is repainted, the text is drawn to the screen and
    the progress bar is updated.
    */
  public void paint(Graphics g) {
    // paint over the old image
    g.setColor(Color.white);
    g.fillRect(0,0,winWidth-1,winHeight-1);

    g.setColor(Color.black);

    g.drawString(line1,10,55);
    g.drawString(line2,10,70);
    g.drawString(line3,10,85);
    g.drawString(line4,10,100);
    g.drawString(line5,10,115);

    bar.percent = (float) i / (float) numCursorBytes;
    border.drawBorder(g);
    bar.drawBar(g);
  }
}

