package evote.util.genrand;

/*
 * KeyboardGenerator.java
 * Last Revised:  5 March 99
 * Brandon DuRette <brandond@mit.edu>
 */

import evote.util.constants.*;
import evote.crypto.*

public class KeyboardGenerator implements Generator {

  /**
   * The KeyboardGenerator class generates an irreproducible seed by performing
   * a secure hash on timing data from the keyboard.
   */

  public KeyboardGenerator() {
    // effects:  Empty constructor, nothing to do.
  }

  public synchronized byte[] seed() {
    // effects:  Prompts user to input keyboard data, times keyboard events, and
    //           uses the timing data to construct a byte array of seed data.

    System.out.println("Begin inputting data from the keyboard. \n\n");
    
    // BWD:  This name no longer relevant, but it works.
    int numBytes = Constants.genrandNumCursorBytes; 
    
    byte[] timingData = new byte[numBytes];
    long oldTime = System.currentTimeMillis();
    long newTime = 0;

    for (int i = 0; i < numBytes; i++) {
      // get data for each byte
      while(System.in.available() == 0) {}
      newTime = System.currentTimeMillis();
      timingData[i] = (byte) (oldTime - newTime);
      oldTime = newTime;
    }

    SHADigest md = new SHADigest();
    byte hashOut[] = md.performHash(gcb.mouseData);
    return hashOut;
  }

  public static void main(String argv[]) {
    KeyboardGenerator keyGen = new KeyboardGenerator();
    
    byte[] bytes = keyGen.seed();
    system.out.write(bytes);
    system.out.flush()
  }
} // End of class KeyboardGenerator
