package evote.voter;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.math.BigInteger;
import java.net.*;
import java.security.*;

import evote.commissioner.complaints.*;
import evote.crypto.*;
import evote.election.*;
import evote.election.viewer.*;
import evote.lang.*;
import evote.network.message.*;
import evote.network.server.*;
import evote.util.constants.*;
import evote.util.genrand.*;
import evote.util.quick_message.*;
import evote.question.*;


/**
  This is the voting "booth" applet that the voter will download and use.  It
  read in an election object and then creates a connection to first the admin
  server and then the anonymizer.
  Revelant numbers, messages and errors will be shown in the appropriate areas
  of the applet.
 */
public class Booth extends Applet implements ActionListener, YesNoCaller, RandomDataRequestor {
  int textWidth = 40;
  int textDepth = 10;
  TextField name_field = new TextField(textWidth);
  TextField password_field = new TextField(textWidth);
  /* Error notification area */
  TextArea error_field = new TextArea(2,textWidth);
  /* Comment area */
  TextArea message_field = new TextArea(8,textWidth);
  /* Calculation result Area */
  TextArea number_field = new TextArea(6,textWidth);
  Dimension win_dimension = new Dimension(400,800);
  Button Redo = new Button("Redo");
  Button logonButton;
  Button yesButton, noButton;

  /* Status Text Field */
  private TextField status_field;

  public static final String VOTER_ELECTION_FILE= "Voter.election";

  /* Card Layout */
  private Panel centerPanel;
  private CardLayout myCardLayout;

  Election election;
  // ElectionViewer ev;
  QuestionListViewer qlv;
  evote.lang.Choice myChoice;
  Vote myVote;
  Vote adminVote;
  GenRandPanel RNG = new GenRandPanel();
  String name;
  String pwd;
  boolean same, good_sig;
  VoteMessage adminSigned;
  RsaEncryption adminSign;
  RsaBlind myBlind;
  String output_message;
  String error_message;
  int error_type = 0;
  boolean questionsReceived= false;

  // the question list used
  private QuestionList myQuestionList;
  private byte[] questionListSignature;

  String adminServer;
  int adminReceivePort;
  BigInteger adminPublicKey = Constants.adminPublic;
  BigInteger adminModulus = Constants.adminMod;
  RsaKeys adminKeys = new RsaKeys(adminPublicKey, adminModulus);

  String anonServer = Constants.anonServer;
  int anonReceivePort = Constants.anonReceivePort;
  BigInteger anonPublicKey = Constants.anonPublic;
  BigInteger anonModulus = Constants.anonMod;
  RsaKeys anonKeys = new RsaKeys(anonPublicKey, anonModulus);

  String counterServer = Constants.counterServer;
  int counterReceivePort = Constants.counterReceivePort;
  BigInteger counterPublicKey = Constants.counterPublic;
  BigInteger counterModulus = Constants.counterMod;
  RsaKeys counterKeys = new RsaKeys(counterPublicKey, counterModulus);

  String commisServer = Constants.commisServer;
  int commisReceivePort = Constants.commisReceivePort;
  BigInteger commisPublicKey = Constants.commisPublic;
  BigInteger commisModulus = Constants.commisMod;
  RsaKeys commisKeys = new RsaKeys(commisPublicKey, commisModulus);

  /* Progress */
  private int progress= NOTHING;
  private static final int NOTHING=0;
  private static final int SIGNED=1;
  private static final int SENT=2;
  /**
    Creates the applet with the appropriate fields and buttons.
   */

  private String paramName, paramPassword;
  private boolean confirmPanelCreated=false;

  public void setStatus(String str) {
    status_field.setText(str);
  }

  public void init() {
    this.setBackground(Color.white);

    Redo.addActionListener(this);
    paramName= getParameter("username");
    paramPassword= getParameter("password");
    
    try {
      /* Load up the Election Object */
      URL voterConfigURL= new URL(getDocumentBase(),VOTER_ELECTION_FILE);
      InputStream is= voterConfigURL.openStream();
      ObjectInputStream ois= new ObjectInputStream(is);
      election= (Election)ois.readObject();
      Constants.init(election);
    } catch(Exception exc) {
      /* Do something drastic if an exception is encountered
       */
      System.out.println("Error loading up election object: " + exc.toString());
    }
    

    adminServer = Constants.adminServer;
    adminReceivePort = Constants.adminReceivePort;
    adminPublicKey = Constants.adminPublic;
    adminModulus = Constants.adminMod;
    adminKeys = new RsaKeys(adminPublicKey, adminModulus);
    
    anonServer = Constants.anonServer;
    anonReceivePort = Constants.anonReceivePort;
    anonPublicKey = Constants.anonPublic;
    anonModulus = Constants.anonMod;
    anonKeys = new RsaKeys(anonPublicKey, anonModulus);
    
    counterServer = Constants.counterServer;
    counterReceivePort = Constants.counterReceivePort;
    counterPublicKey = Constants.counterPublic;
    counterModulus = Constants.counterMod;
    counterKeys = new RsaKeys(counterPublicKey, counterModulus);

    commisServer = Constants.commisServer;
    commisReceivePort = Constants.commisReceivePort;
    commisPublicKey = Constants.commisPublic;
    commisModulus = Constants.commisMod;
    commisKeys = new RsaKeys(commisPublicKey, commisModulus);

    adminSign = new RsaEncryption(adminPublicKey, adminModulus);

    Panel logonPanel= new Panel();
    Panel loggingOnPanel= new Panel();
    Panel questionPanel= new Panel();
    centerPanel= new Panel();
    myCardLayout= new CardLayout();
    centerPanel.setLayout(myCardLayout);

    loggingOnPanel.add(new Label("Logging you on to EVOX, please wait..."));

    logonButton= new Button("Log On");
    logonButton.addActionListener(this);

    name_field.setEditable(true);
    message_field.setEditable(false);

    error_field.setEditable(false);

    number_field.setEditable(false);
    // number_field.resize(error_field.preferredSize());

    password_field.setEditable(true);
    // password_field.resize(password_field.preferredSize());
    password_field.setEchoChar('*');

    this.setLayout(new BorderLayout());
    
    logonPanel.setLayout(new GridLayout(3,1));

    Panel namePanel= new Panel();
    namePanel.add(new Label("Name:",Label.CENTER));
    namePanel.add(name_field);
    Panel pwdPanel= new Panel();
    pwdPanel.add(new Label("Password:", Label.CENTER));
    pwdPanel.add(password_field);
    Panel buttonPanel= new Panel();
    buttonPanel.add(logonButton);

    logonPanel.add(namePanel);
    logonPanel.add(pwdPanel);
    logonPanel.add(buttonPanel);

    centerPanel.add("LOGON",logonPanel);
    centerPanel.add("LOGGINGON", loggingOnPanel);

    status_field= new TextField();
    status_field.setEditable(false);

    // ev = new ElectionViewer(win_dimension, election, this);    

    this.add("Center",centerPanel);
    this.add("South",status_field);

    Panel randomPanel= RNG.getPanel();

    centerPanel.add("RANDOM", randomPanel);
    myCardLayout.show(centerPanel,"RANDOM");
    RNG.seed(this);

    System.out.println("seeded");
  } /* init */


  public void randomDataGenerated() {
    myCardLayout.show(centerPanel,"LOGON");

    if ((paramName!=null)&&(paramPassword!=null)) {
      name_field.setText(paramName);
      password_field.setText(paramPassword);
      doLogon();
    }
  }    

  /**
    This method performs the logon
    */
  public void doLogon() {
    
    status_field.setText("");
    myCardLayout.show(centerPanel, "LOGGINGON");

    name= name_field.getText();
    pwd= password_field.getText();
    
    VoterIdentityObject vido= new VoterIdentityObject(name, pwd);

    // Create a Logon Message
    LogonMessage lm= new LogonMessage(vido);
    
    try {
      // Connect to the Admin
      SecureConnection adminConnection = new SecureConnection(Constants.adminServer, Constants.adminReceivePort, RNG, adminKeys);

      // send logon message
      adminConnection.sendMessage(lm);
      
      // get response
      Message response= adminConnection.receiveMessage();

      // if it's a question list, cool!
      if (response instanceof QuestionListMessage) {
	// verify the signature on the question List
	QuestionListMessage qlm= (QuestionListMessage)response;
	QuestionList myQL= qlm.getQuestionList();
	byte[] QLBytes= myQL.serialize();
	ShaCommitmentKeys sha_keys= new ShaCommitmentKeys(new BigInteger("0"), new BigInteger("0"));
	ShaCommitment sha= new ShaCommitment(sha_keys);
	byte[] receivedBytes= sha.commit(QLBytes);

	if (adminSign.verify(receivedBytes, qlm.getSignature())) {
	  myQuestionList= myQL;
	  questionListSignature= qlm.getSignature();
	  switchToQuestions(myQL);
	}
	else {
	  status_field.setText("Question List NOT PROPERLY SIGNED! Intrusion Attempt!");
	}
	
	return;
      }

      // if it's an error
      if (response instanceof ErrorMessage) {
	status_field.setText("Error Logging On. Check Username and Password");
	myCardLayout.show(centerPanel, "LOGON");	
	return;
      }
    } 
    catch(Exception exc) {
      status_field.setText("Error connecting to the Admin " + adminServer + ": " + exc.toString());
      gotoError();
      // exc.printStackTrace();
    }
  }

  /**
    This switches the screen to Question Answering Mode
    */
  private void switchToQuestions(QuestionList ql) {
    questionsReceived= true;
    
    // This code is for multiple responses to each
    // question.
    /*
      MultipleChoice mult_choice= new MultipleChoice(ql.getNumQuestions());
      for (int i=0; i<ql.getNumQuestions(); i++) {
      // hack
      // mult_choice.setQuestionSize(i, ql.getQuestion(i).getNumAnswers());
      mult_choice.setQuestionSize(i, 3);
      }
      */
    
    evote.lang.Choice single_choice= new evote.lang.Choice(ql.getNumQuestions());

    qlv= new QuestionListViewer(ql, this, single_choice);
    
    centerPanel.add("VOTE", qlv);
    myCardLayout.show(centerPanel, "VOTE");
  }

  /**
    This is the method that does all the work once the voters choices have been
    recorded.  It opens the appropriate SecureConnections and checks that
    all information from the servers are correct.  It prints appropriate
    messages to varioes applet areas.
   */
  public void goVote() {
    /* Create a quickie Panel */
    if (!confirmPanelCreated) {
      Panel confirmPanel= new Panel();
      confirmPanel.setLayout(new BorderLayout());
      
      Panel topPanel= new Panel(new GridLayout(2,1));
      topPanel.add(new Label("Are you sure you want to send your vote?"));
      topPanel.add(new Label("You will NOT be able to modify it later!"));
      
      confirmPanel.add("North",topPanel);
      
      Panel buttonPanel= new Panel(new FlowLayout());
      yesButton= new Button("Send the Vote");
      noButton= new Button("Don't send the Vote Yet");

      yesButton.addActionListener(this);
      noButton.addActionListener(this);

      buttonPanel.add(yesButton);
      buttonPanel.add(noButton);

      confirmPanel.add("South", buttonPanel);

      centerPanel.add("CONFIRMVOTE", confirmPanel);
      confirmPanelCreated= true;
    }

    myCardLayout.show(centerPanel,"CONFIRMVOTE");
  }

  public void doNo() {
    myCardLayout.show(centerPanel,"VOTE");
  }
  
  public void doYes() {

    /* Check Progress */
    try { /* everything, to catch all complaints */
      
      if (progress < SIGNED) {
	name = name_field.getText();
	pwd  = password_field.getText();
	VoterIdentityObject vido = new VoterIdentityObject(name, pwd);
	
	status_field.setText("Processing Vote");
	
	try {
	  myVote = new Vote(myChoice);
	} catch (Exception exp) {
	  error_type = 13;
	  error_message = new String("Problem creating (Vote) myVote: " + exp.toString());
	  status_field.setText("Error with Vote: " + exp.toString());
	  throw (new Exception(error_message));
	}
	
	status_field.setText("Committing Vote");
	
	ShaCommitment myCommit = new ShaCommitment(RNG);
	myVote.commit(myCommit);
	
	status_field.setText("Blinding Vote");
	
	try {
	  myBlind = new RsaBlind(adminPublicKey, adminModulus, RNG);
	  myVote.blind(myBlind);
	} catch (Exception exp) {
	  error_type = 12;
	  error_message = new String("There was a " + exp.toString() + " exception while blinding." + "\n");
	  
	  status_field.setText("Error Blinding Vote: " + exp.toString());
	  
	  throw (new Exception(error_message));
	}

	status_field.setText("Making Vote Admin-Safe");
	
	try {
	  adminVote = myVote.makeAdminSafe();
	} catch (Exception exp) {
	  error_type = 13;
	  error_message = new String("There was a " + exp.toString() + " exception while trying to make the vote admin safe." + "\n");
	  error_field.appendText(error_message);
	  status_field.setText("Error making Vote admin-safe: " + exp.toString());
	  throw (new Exception(error_message));
	}
	
	status_field.setText("Sending Vote to Admin");
	
	VoteMessage send_to_admin = new VoteMessage(adminVote, vido);
	
	Message message_from_admin = null;
	
	try {
	  // message_field.appendText("Creating a new secure connection to admin server...");
	  SecureConnection adminConnection = new SecureConnection(adminServer, Constants.adminReceivePort, RNG, adminKeys);
	  adminConnection.sendMessage(send_to_admin);
	  
	  status_field.setText("Sent Vote to Admin");
	  
	  message_from_admin = adminConnection.receiveMessage();
	  
	  status_field.setText("Received Admin Response");
	  
	  adminConnection.close();
	}
	catch(Exception exp) {
	  error_type = 1;
	  error_message = new String("There was a " + exp.toString() + " exception thrown during the connection to the admin server." + "\n");
	  
	  status_field.setText("Error talking to Admin Server");
	  
	exp.printStackTrace();

	  throw (new Exception(error_message));
	}
	
	
	good_sig= false;
	
	if (message_from_admin instanceof VoteMessage) {
	  adminSigned = (VoteMessage)message_from_admin;
	  adminVote = adminSigned.getVote();
	} else {
	  if (message_from_admin instanceof ComplaintMessage) {
	    ComplaintMessage cm = (ComplaintMessage)message_from_admin;
	    status_field.setText("Error: " + cm.getComplaint().getProblemString());
	    throw(new Exception("bad password"));
	  } else {
	    if (message_from_admin instanceof ErrorMessage) {
	      gotoError("It seems that you have already voted.\n You are only allowed to vote once!\n");
	      return;
	    }
	    error_type = 3;
	    
	    status_field.setText("Error: Unknown Admin Return Message");
	    
	    throw (new Exception("bad message"));
	  }
	}
	
	System.out.println("got vote from Admin");
	
	try {
	  myVote.getSignature(adminVote);
	  
	  status_field.setText("Got Admin Signature");
	  
	} catch (Exception exp) {
	  error_type = 13;
	  throw (new Exception("Could not get Admin Signature"));
	}
	
	System.out.println("Got signature");
		
	try {
	  myVote.unblind(myBlind);
	  status_field.setText("Unblinding Admin Signature");
	} catch (Exception exp) {
	  error_type = 13;
	  throw (new Exception("Error unblinding admin signature"));
	}
	
	System.out.println("unblind");	

	
	try {
	  status_field.setText("Verifying Admin Signature");
	  good_sig = myVote.verifyAdminSign(adminSign);
	} catch (Exception exp) {
	  error_type = 10;
	  status_field.setText("Error in Admin Signature");
	  throw (new Exception("Error in Admin Signature"));
	}
	
	
	if (!good_sig) {
	  error_type = 10;
	  status_field.setText("Invalid Admin Signature");
	  throw (new Exception("Invalid Admin Signature"));
	}
	
	// Mark the progress we've made
	progress= SIGNED;
      }

      
      if (progress < SENT) {
	try {
	  Message message_from_anon = null;
	  status_field.setText("Attempting Connection to Anonymizer");
	  SecureConnection SecureConnectionToAnon = new SecureConnection(anonServer, anonReceivePort, RNG, anonKeys);
	  SecureConnection SecureConnectionToCounter = new SecureConnection(SecureConnectionToAnon, RNG, counterKeys);
	  status_field.setText("Making Vote Safe for Counter");
	  Vote counterVote = myVote.makeCounterSafe();
	  counterVote.setQuestionList(myQuestionList, questionListSignature);
	  VoteMessage send_to_anon = new VoteMessage(counterVote);
	  status_field.setText("Sending Vote to Anonymizer");
	  SecureConnectionToCounter.sendMessage(send_to_anon);
	  
	  status_field.setText("Receiving Anonymizer Response");
	  message_from_anon = SecureConnectionToAnon.receiveMessage();
	  SecureConnectionToAnon.close();
	  
	  if (message_from_anon instanceof OKMessage) {
	    status_field.setText("Response from Anonymizer is Okay. Voting Complete");
	  } else {
	    System.out.println("Problem with return message from Anon");
	    error_type = 3;
	    status_field.setText("Bad Response from Anonymizer");
	    throw (new Exception("Bad Response from Anon"));
	  }
	  
	  
	} catch (Exception exp) {
	  exp.printStackTrace();
	  error_type = 1;
	  status_field.setText("Bad connection to Anonymizer");
	  throw (new Exception("Bad Connection to Anonymizer"));
	}
      
	progress= SENT;
      }
      doneVoting();
	
    } catch (Exception exp) {
      if (error_type == 0) {
	error_type = 13;
      }
      Complaint comp = new Complaint(name, "" , error_type, exp.toString(), RNG);
      ComplaintMessage compMsg = new ComplaintMessage(comp);
      try {
	SecureConnection commisConnection = new SecureConnection(commisServer, Constants.commisReceivePort, RNG, commisKeys);
	commisConnection.sendMessage(compMsg);
      } catch (Exception e) {
	status_field.setText("Could not contact Commissioner! Please Save All Text and Report error in person");
      }

      gotoError();
    }
  }
  

  public void gotoError() {
    /* Put up a screen that says that that something went wrong */
    
    String problem= "";
    problem += "Chances are that one of the EVOX servers is currently too busy\n";
    problem += "to handle your request. Please wait a few seconds and try again\n";      
    problem += "by clicking on the button below.\n\n";
    problem += "If you have already filled out the ballot\n";
    problem += "DO NOT QUIT YOUR BROWSER! For security reasons, you would then\n";
    problem += "be unable to vote again.\n";

    gotoError(problem);
  }    
  
  public void gotoError(String message) {
    /* Put up a screen that says that that something went wrong */
    Panel problemPanel= new Panel();
    problemPanel.setLayout(new BorderLayout());
    
    Panel thisCenterPanel= new Panel();
    TextArea problemArea= new TextArea("", 15,60,TextArea.SCROLLBARS_NONE);
    problemArea.append("There was an error while processing your vote.\n");
    problemArea.append(message);
    problemArea.setEditable(false);
    thisCenterPanel.add(problemArea);
    
    Panel thisButtonPanel= new Panel();
    thisButtonPanel.add(Redo);
    problemPanel.add("Center", thisCenterPanel);
    problemPanel.add("South",thisButtonPanel);
    
    centerPanel.add("PROBLEM", problemPanel);
    myCardLayout.show(centerPanel, "PROBLEM");
  }

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();

    if (source == yesButton) {
      doYes();
      return;
    }

    if (source == noButton) {
      doNo();
      return;
    }

    if (source == logonButton) {
      // System.err.println("HELLO???");
      doLogon();
      return;
    }

    if (source == Redo) {
      status_field.setText("");
      if (questionsReceived) {
	doYes();
      } else {
	doLogon();
      }
    }
  }


  /**
    The Election button (click) that sets it all in motion.
   */
  public void done() {
    myChoice = qlv.getChoice();
    goVote();           
  }

  private void doneVoting() {
    /* Prepare the Done panel */
    Panel donePanel= new Panel();
    donePanel.add(new Label("Thank you for voting. Your vote has been successfully recorded."));

    centerPanel.add("DONE", donePanel);
    myCardLayout.show(centerPanel, "DONE");
  }

} /* public class Booth extends Applet implements ActionListener */

