/*
 * Commissioner Setup
 * This panel allows a user to
 * choose the various
 */

package evote.commissioner;

import evote.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;

public class CommissionerSetupPanel extends Panel implements ActionListener {

  private CommissionerParameters myParameters;
  private TextField hostnameField, portField, numThreadsField, queueSizeField;
  private Button commitButton;

  private boolean editable= true;

  public CommissionerSetupPanel() {
    myParameters= new CommissionerParameters();
    init();
  }

  public CommissionerSetupPanel(CommissionerParameters vp) {
    myParameters= vp;
    init();
  }

  private void init() {
    hostnameField= new TextField(20);
    portField= new TextField(5);
    numThreadsField= new TextField(5);
    queueSizeField= new TextField(5);

    String currentHostName= myParameters.getHostName();
    if (currentHostName != null)
      hostnameField.setText(currentHostName);
    
    int currentPort= myParameters.getPort();
    if (currentPort > 0)
      portField.setText("" + currentPort);

    int num_threads= myParameters.getNumThreads();
    if (num_threads > 0)
      numThreadsField.setText("" + num_threads);
    
    int queue_size= myParameters.getQueueSize();
    queueSizeField.setText("" + queue_size);

    this.setLayout(new GridLayout(5,1));

    Panel topPanel= new Panel();
    topPanel.add(new Label("Hostname:"));
    topPanel.add(hostnameField);
    
    Panel bottomPanel= new Panel();
    bottomPanel.add(new Label("Port:"));
    bottomPanel.add(portField);

    Panel buttonPanel= new Panel();
    commitButton= new Button("Commit");
    buttonPanel.add(commitButton);
    commitButton.addActionListener(this);

    Panel threadsPanel= new Panel();
    threadsPanel.add(new Label("Number of Threads:"));
    threadsPanel.add(numThreadsField);

    Panel queuePanel= new Panel();
    queuePanel.add(new Label("Size of Server Queue:"));
    queuePanel.add(queueSizeField);

    this.add(topPanel);
    this.add(bottomPanel);
    this.add(threadsPanel);
    this.add(queuePanel);
    this.add(buttonPanel);
  }

  
  public void actionPerformed(ActionEvent ae) {
    
    if (editable) {
      try {
	String hostname= hostnameField.getText();
	int port= Integer.parseInt(portField.getText());
	
	InetAddress check= InetAddress.getByName(hostname);
	
	myParameters.setLocation(hostname, port);

	int num_threads= Integer.parseInt(numThreadsField.getText());
	int queue_size= Integer.parseInt(queueSizeField.getText());

	myParameters.setNumThreads(num_threads);
	myParameters.setQueueSize(queue_size);
	
	hostnameField.setEditable(false);
	portField.setEditable(false);
	numThreadsField.setEditable(false);
	queueSizeField.setEditable(false);
	commitButton.setLabel(" Edit ");
	editable= false;
      }
      catch(Exception exc) {
      }
    } else {
      hostnameField.setEditable(true);
      portField.setEditable(true);
      numThreadsField.setEditable(false);
      queueSizeField.setEditable(false);
      commitButton.setLabel("Commit");
      editable=true;
    }

  }

}
