package evote.commissioner.complaints;

import java.io.*;
import java.security.*;
import java.util.Date;

import evote.crypto.Signer;
import evote.network.server.*;
import evote.util.constants.*;
import evote.util.genrand.*;


/**
  A complaint is a message from a party participating in the election to the
  commissioner, and possibly another party as well.
  */
public class Complaint implements Serializable {
  public String plaintif;
  public String defendant;
  public Date time_complaint_made;
  public Date time_complaint_received;
  int problem;
  String description = "";

  public static final int Connection_Error = 1;
  public static final int Bad_MAC = 2;
  public static final int Bad_Message_Format = 3;
  public static final int No_Keys = 4;
  public static final int Voter_Not_Registered = 5;
  public static final int Bad_Password = 6;
  public static final int Already_Signed = 7;
  public static final int Already_committed = 8;
  public static final int Already_Uncommitted = 9;
  public static final int Bad_Sig = 10;
  public static final int File_Error = 11;
  public static final int Math_Error = 12;
  public static final int Unknown_Error = 13;
  public static final int After_Deadline = 14;
  public static final int Already_Voted = 15;
  public static final int Questionlist_Invalid= 16;

  public Complaint(String plntf, String dfnd, int prob, GenRand RNG) {
    time_complaint_made = new Date();
    plaintif = plntf;
    defendant = dfnd;
    problem = prob;
  }

  public Complaint(String plntf, String dfnd, int prob, String dscpt, GenRand RNG) {
    time_complaint_made = new Date();
    plaintif = plntf;
    defendant = dfnd;
    problem = prob;
    description = dscpt;
  }

  public Complaint(String plntf, String dfnd, int prob, GenRand RNG, Signer s) {
    time_complaint_made = new Date();
    plaintif = plntf;
    defendant = dfnd;
    problem = prob;
  }

  public Complaint(String plntf, String dfnd, int prob, String dscpt, GenRand RNG, Signer s) {
    time_complaint_made = new Date();
    plaintif = plntf;
    defendant = dfnd;
    problem = prob;
    description = dscpt;
  }

  public void setTimeRcvd() {
    time_complaint_received = new Date();
  }

  public String getPlaintif() {
    return plaintif;
  }

  public String getDefendant() {
    return defendant;
  }

  public int getProblemInt() {
    return problem;
  }

  public String getProblemString() {
    String prob_text;
    switch (problem) {
    case  1: prob_text = new String("connection error"); break;
    case  2: prob_text = new String("bad MAC"); break;
    case  3: prob_text = new String("bad message"); break;
    case  4: prob_text = new String("no keys"); break;
    case  5: prob_text = new String("voter not registered"); break;
    case  6: prob_text = new String("bad password"); break;
    case  7: prob_text = new String("already signed"); break;
    case  8: prob_text = new String("already committed"); break;
    case  9: prob_text = new String("already uncommitted"); break;
    case 10: prob_text = new String("bad signature"); break;
    case 11: prob_text = new String("file error"); break;
    case 12: prob_text = new String("math error"); break;
    case 13: prob_text = new String("unkown error"); break;
    case 14: prob_text = new String("after deadline"); break;
    case 15: prob_text = new String("voter already voted"); break;
    default: prob_text = new String("problem not listed"); break;
    }
    return prob_text;
  }

  public Date getTimeMade() {
    return time_complaint_made;
  }

  public Date getTimeReceived() {
    return time_complaint_received;
  }

  public String getDescription() {
    return description;
  }


  public byte[] serialize() throws Exception {
    byte[] ser_comp;
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    try {
      ObjectOutputStream oos = new ObjectOutputStream(baos);
      oos.writeObject(this);
      ser_comp= baos.toByteArray();
      return(ser_comp);
    }
    catch(IOException ie) {
      throw(new Exception("couldn't serialize"));
    }
  }

  public static Complaint deserialize(byte[] b) throws Exception {
    try {
      ByteArrayInputStream bais = new ByteArrayInputStream(b);
      ObjectInputStream ois = new ObjectInputStream(bais);
      Complaint c = (Complaint)ois.readObject();
      return(c);
    }
    catch(Exception exp) {
      throw(new Exception("Format Error"));
    }
  }

}
