/* Electronic Voting Project
 * Ben Adida
 * Counter Interface 
 */

package evote.counter;

import java.awt.*;
import java.awt.event.*;
import evote.counter.*;
import java.util.*;
import evote.election.*;
import java.math.*;
import java.io.*;
import evote.lang.*;


/**
 * The Counter Interface
 *
 * This is a simple GUI to the Counter.
 * 
 * @see Counter
 */
public class CounterInterface extends Frame implements WindowListener, ActionListener, ItemListener, VoteListener {
  
  private Button count_votes_button, quit_button;
  private List vote_list;
  private TextField num_votes_field;
  private Vector votes_received;
  private Election my_election;
  private Counter myCounter;
  
  /** 
   * This creates the Counter Interface
   * 
   * @argument e The Election that is being held.
   */
  public CounterInterface(Election e, Counter c){
    super("Vote Counter");
    
    this.addWindowListener(this);
    
    votes_received= new Vector();

    my_election= e;
    myCounter= c;

    initGUI();
    this.pack();
    this.setSize(300,400);
    this.show();
  }
    
  private void initGUI() {

    count_votes_button= new Button("Count Votes");
    count_votes_button.addActionListener(this);
    quit_button= new Button("Quit");
    quit_button.addActionListener(this);

    vote_list= new List();
    vote_list.addActionListener(this);
    vote_list.addItemListener(this);
    num_votes_field= new TextField();
    num_votes_field.setEditable(false);
    num_votes_field.setText("" + votes_received.size());
    this.setLayout(new BorderLayout());
    this.add("Center",vote_list);
    
    Panel south_p;
    south_p= new Panel();
    south_p.setLayout(new GridLayout(3,1));
    south_p.add(new Label("# Votes Received:", Label.CENTER));
    south_p.add(num_votes_field);
    
    Panel button_p= new Panel();

    button_p.add(count_votes_button);
    button_p.add(quit_button);
    south_p.add(button_p);
    this.add("South",south_p);
  }

  public synchronized void receiveVote(VoteEvent ve) {
    /* Do something when a vote is received */
    //System.out.println("INTERFACE received a vote ");
    try {
      votes_received.addElement(ve.getVote());

      // vote_list.addItem("Vote #" + votes_received.size() + ": " + new BigInteger(ve.getVote().getCommittedBytes()));

      num_votes_field.setText("" + votes_received.size());
    }
    catch(Exception e) {
      System.out.println("ERROR IN VOTE!!");
    }
  }

  /* This reads in a list of votes from a file */
  public static void main(String argv[]) {
    try {
      FileInputStream fis= new FileInputStream(argv[0]);
      ObjectInputStream ois= new ObjectInputStream(fis);
      
      ElectionLoader eLoader= new ElectionLoader();
      Election myElec= eLoader.getElection();

      CounterInterface mine= new CounterInterface(myElec, null);
      
      try {
	while(true) {
	  Vote v= (Vote)ois.readObject();
	  mine.votes_received.addElement(v);
	  mine.num_votes_field.setText("" + mine.votes_received.size());
	}
      } catch(Exception exc) {
      }

    } catch(Exception exc2) {
      System.err.println("problem: " + exc2.toString());
    }
  }
      
  public void actionPerformed(ActionEvent e) {

    Object source= e.getSource();

    /* Count Votes */
    if (source == count_votes_button) {
      /* count_votes_button.setVisible(false); */
      // HACK FOR NOW
      // SimpleMultipleVoteCounter vc= new SimpleMultipleVoteCounter(my_election);
	// MultipleVoteCounter vc= new MultipleVoteCounter(my_election);

      VoteCounter vc= new VoteCounter(my_election);
      vc.countVotes(votes_received.elements());
      vc.displayResults();
      return;
    }

    /* Quit Button Pressed */
    if (source == quit_button) {
      if (myCounter != null)
	myCounter.reallyQuit();
    }

    /* View a Choice better */
    if (source == vote_list) {
      VoteCounter vc= new VoteCounter(my_election);
      Vector temp_v= new Vector();
      temp_v.addElement(votes_received.elementAt(vote_list.getSelectedIndex()));
      vc.countVotes(temp_v.elements());
      vc.displayResults();
      return;
    }

  }
    
  public void itemStateChanged(ItemEvent e) {
  }

  public void windowActivated(WindowEvent e) {
  }

  public void windowClosed(WindowEvent e) {
  }
  
  public void windowClosing(WindowEvent e) {
  }

  public void windowDeactivated(WindowEvent e) {
  }

  public void windowDeiconified(WindowEvent e) {
  }

  public void windowIconified(WindowEvent e) {
  }

  public void windowOpened(WindowEvent e) {
  }

}
    
		      
