/*
 * Key Setup Panel
 */

package evote.election.builder;

import java.awt.*;
import java.awt.event.*;

import evote.lang.*;
import evote.util.genrand.*;
import evote.counter.*;
import evote.admin.*;
import evote.anonymizer.*;
import evote.crypto.*;
import evote.commissioner.*;

public class KeySetupPanel extends Panel implements ActionListener {

  /* The servers that need keys generated */
  private CounterParameters counterParams;
  private AdminParameters adminParams;
  private AnonymizerParameters anonymizerParams;
  private CommissionerParameters commisParams;

  private GenRand myRNG;
  
  /* GUI Elements */
  private TextField counterField, adminField, anonymizerField, commisField;
  private Button counterButton, adminButton, anonymizerButton, commisButton;

  public KeySetupPanel(CounterParameters count_params,
		       AdminParameters admin_params,
		       AnonymizerParameters anon_params,
		       CommissionerParameters commis_params) {
    this.counterParams= count_params;
    this.adminParams= admin_params;
    this.anonymizerParams= anon_params;
    this.commisParams= commis_params;

    /* Setup GUI */
    counterField= new TextField();
    counterField.setEditable(false);
    adminField= new TextField();
    adminField.setEditable(false);
    anonymizerField= new TextField();
    anonymizerField.setEditable(false);
    commisField= new TextField();
    commisField.setEditable(false);

    checkKeys();

    counterButton= new Button("Generate");
    adminButton= new Button("Generate");
    anonymizerButton= new Button("Generate");
    commisButton= new Button("Generate");

    counterButton.addActionListener(this);
    adminButton.addActionListener(this);
    anonymizerButton.addActionListener(this);
    commisButton.addActionListener(this);

    this.setLayout(new BorderLayout());

    Panel titlePanel= new Panel();
    titlePanel.setLayout(new GridLayout(4,1));
    titlePanel.add(new Label("Counter"));
    titlePanel.add(new Label("Administrator"));
    titlePanel.add(new Label("Anonymizer"));
    titlePanel.add(new Label("Commissioner"));

    Panel buttonPanel= new Panel();
    buttonPanel.setLayout(new GridLayout(4,2));
    buttonPanel.add(counterField);
    buttonPanel.add(counterButton);
    buttonPanel.add(adminField);
    buttonPanel.add(adminButton);
    buttonPanel.add(anonymizerField);
    buttonPanel.add(anonymizerButton);
    buttonPanel.add(commisField);
    buttonPanel.add(commisButton);
    
    this.add("Center",titlePanel);
    this.add("East",buttonPanel);
  }

  private void checkKeys() {
    
    // Counter Keypair
    if (counterParams.getKeys() != null)
      counterField.setText("GENERATED");
    else
      counterField.setText("*MISSING*");
    
    // Admin Keypair
    if (adminParams.getKeys() != null)
      adminField.setText("GENERATED");
    else
      adminField.setText("*MISSING*");

    // Anon keypair
    if (anonymizerParams.getKeys() != null)
      anonymizerField.setText("GENERATED");
    else
      anonymizerField.setText("*MISSING*");

    // Commis keypair
    if (commisParams.getKeys() != null)
      commisField.setText("GENERATED");
    else
      commisField.setText("*MISSING*");
  }

  public void actionPerformed(ActionEvent ae) {
    Object source= ae.getSource();
    
    if (myRNG == null) {
      myRNG= new GenRand();
      myRNG.seed();
    }
    
    RsaKeys new_rsk= new RsaKeys(myRNG);

    RsaKeys new_rsk_public= new RsaKeys(new_rsk.getRsaE(), new_rsk.getRsaN());

    if (source == counterButton) {
      counterParams.setKeys(new_rsk);
      adminParams.setCounterKeys(new_rsk_public);
      anonymizerParams.setCounterKeys(new_rsk_public);
      commisParams.setCounterKeys(new_rsk_public);
    }

    if (source == adminButton) {
      adminParams.setKeys(new_rsk);
      anonymizerParams.setAdminKeys(new_rsk_public);
      counterParams.setAdminKeys(new_rsk_public);
      commisParams.setAdminKeys(new_rsk_public);
    }

    if (source == anonymizerButton) {
      anonymizerParams.setKeys(new_rsk);
      adminParams.setAnonKeys(new_rsk_public);
      counterParams.setAnonKeys(new_rsk_public);
      commisParams.setAnonKeys(new_rsk_public);
    }

    if (source == commisButton) {
      commisParams.setKeys(new_rsk);
      adminParams.setCommisKeys(new_rsk_public);
      counterParams.setCommisKeys(new_rsk_public);
      anonymizerParams.setCommisKeys(new_rsk_public);
    }

    checkKeys();
  }

}

