/*
 * ElectionViewer.java
 * Last Revised:  13 May 97
 * Randy L. Milbert <rmilbert@mit.edu>
 */

package evote.election.viewer;

import evote.util.quick_message.QuickMessage;
import evote.election.*;
import evote.lang.Choice;
import evote.voter.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 *  The ElectionViewer panel provides an interface that the voter
 *  may use to make his or her selections.
 */

public class ElectionViewer extends Panel {
	private	Dimension			win_dimension;
	private	Election			election;
	private	QuestionPanel		question_panel[];
	private	int					num_questions;
	private	Choice				choice;
  private Booth parent;

  /**
   *  The constructor takes three objects as arguments:  Dimension, Election,
   *  and Booth.  The Dimension object specifies the height and width of the
   *  panel.  The Election object contains the questions and choices of the
   *  ballot.  The Booth object is notified when all of the selections have
   *  been made.
   */

	public ElectionViewer(Dimension win_dimension, Election election, Booth parent) {
		this.win_dimension	= win_dimension;
		num_questions		= election.getNumQuestions();
		this.parent = parent;

		// setSize(win_dimension);
		setLayout(new CardLayout());
		setBackground(Color.lightGray);
		
		choice = new Choice(num_questions);

		question_panel = new QuestionPanel[num_questions];

		for (int i = 0; i < num_questions; i++) {
			question_panel[i] = new QuestionPanel(this, win_dimension, election, choice, i);
		}
	}

  /**
   *  The getChoice() method returns the Choice object generated by the
   *  user's selections.
   */

  public Choice getChoice() {
    return choice;
  }

  /**
   *  The done() method is called when the user is finished making his
   *  or her selections.
   */

  public void done() {
    parent.done();
  }
}
