/*
 * User class
 * defines a user, with full name, email address, password
 */

package evote.identity;

import java.io.*;

public class User implements Serializable {
  private String lastName, firstName;
  private String userName;
  private String emailAddress;
  private String password;
  private boolean has_voted= false;

  /* Normal Constructor */
  public User(String uname, String ln, String fn, String ea) {
    this.userName= uname;
    this.lastName= ln;
    this.firstName= fn;
    this.emailAddress= ea;
  }
	
  /* This changes the info, given the old password */
  public boolean edit(String ln, String fn, String ea, String old_pwd) {
    if (verify(old_pwd)) {
      this.lastName= ln;
      this.firstName= fn;
      this.emailAddress= ea;
      return(true);
    }
    return(false);
  }

  /* This attempts to set a new password, 
   * only if the old password is given correctly.
   * If the password hasn't yet been set, then old_pwd
   * must be null
   */
  public boolean setPassword(String old_pwd, String new_pwd) {
    if ((old_pwd == password) || (old_pwd.equals(password))) {
      password= new_pwd;
      return(true);
    }

    return(false);
  }

  /* Verifies the password
   * returns a boolean
   */
  public final boolean verify(String pwd) {
    if ((password == pwd) || (pwd.equals(password)))
      return(true);
    else
      return(false);
  }

  public void voted() {
    has_voted= true;
  }

  public boolean hasVoted() {
    return(has_voted);
  }

  protected void clear() {
    has_voted= false;
  }

  /* Returns username */
  public String getUserName() {
    return(userName);
  }

  /* Return stuff */
  public String getLastName() {
    return(lastName);
  }

  public String getFirstName() {
    return(firstName);
  }

  public String getEmailAddress() {
    return(emailAddress);
  }

}
