/*
 * The GUI that displays the time before starting
 * and the time to stop
 */

package evote.lang;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;


public class TimeControllerGUI extends Frame {

  private TextField nextTimeField, nextActionField, currentStatusField;
  private Date startTime, endTime, nextTime;
  private Button quitButton;
  private DateFormat dateFormat;
  
  public TimeControllerGUI(Date sTime, Date eTime) {
    super("Time Controller");
    startTime= sTime;
    endTime= eTime;
    nextTime= startTime;
    
    /* Set up the GUI */
    nextTimeField= new TextField();
    nextTimeField.setEditable(false);
    nextActionField= new TextField();
    nextActionField.setEditable(false);
    currentStatusField= new TextField();
    currentStatusField.setEditable(false);

    nextActionField.setText("Current Time:");

    quitButton= new Button("Quit");
    quitButton.setEnabled(false);

    this.setLayout(new BorderLayout());
    
    Panel centerPanel= new Panel();
    centerPanel.setLayout(new GridLayout(2,2));

    dateFormat= DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM);

    /* set up the center panel in a 2x2 grid */
    centerPanel.add(new Label("START:" + dateFormat.format(startTime),Label.CENTER));
    centerPanel.add(nextActionField);
    centerPanel.add(new Label("END  :" + dateFormat.format(endTime),Label.CENTER));
    centerPanel.add(nextTimeField);
    
    Panel bottomPanel= new Panel();
    bottomPanel.add(quitButton);
    
    this.add("South", bottomPanel);
    this.add("North", currentStatusField);
    this.add("Center", centerPanel);
    
    this.pack();
    this.setSize(400,150);
    this.show();

    currentStatusField.setText("Waiting for Start Time");
  }

  public void update(Date now) {
    nextTimeField.setText(dateFormat.format(now));
  }

  public void started() {
    nextTime= endTime;
    currentStatusField.setText("System Running.");
  }

  public void stopped() {
    currentStatusField.setText("System Shutting Down.");
  }

}		    

