package evote.util.genrand;

/*
 * KeyboardGenerator.java
 * Last Revised:  5 March 99
 * Brandon DuRette <brandond@mit.edu>
 */

import java.io.*;
import evote.util.constants.*;
import evote.crypto.*;

public class KeyboardGenerator implements Generator {

  /**
   * The KeyboardGenerator class generates an irreproducible seed by performing
   * a secure hash on timing data from the keyboard.
   */

  private static int NUM_STRINGS = 30;
  private static int MAX_BYTES   = 20;

  public KeyboardGenerator() {
    // effects:  Empty constructor, nothing to do.
  }

  public synchronized byte[] seed() {
    // effects:  Prompts user to input keyboard data, times keyboard events, and
    //           uses the timing data to construct a byte array of seed data.

    System.out.println("Begin inputting data from the keyboard. \n\n");
    
    long oldTime = System.currentTimeMillis();
    long newTime = 0;

    byte[] timingData = new byte[NUM_STRINGS];
    byte[] seed       = new byte[0];
    byte[] string     = new byte[MAX_BYTES];

    SHADigest md = new SHADigest();

    for (int i = 0; i < NUM_STRINGS; i++) {
      // get data for each byte

      try {
	System.in.read(string);
	newTime = System.currentTimeMillis();
	timingData[i] = (byte) (newTime - oldTime);
	oldTime = newTime;
	
	// Hash the bytes coming in.
	byte[] toHash = new byte[seed.length + string.length + timingData.length];
	System.arraycopy(seed, 0, toHash, 0, seed.length);
	System.arraycopy(string, 0, toHash, seed.length, string.length);
	System.arraycopy(timingData, 0, toHash, seed.length+string.length, 
			timingData.length);
	
	seed = md.performHash(toHash);

      } catch (IOException e) {
	// This is probably really bad.
	System.err.println("ERROR:  No keyboard data can be read!");
      }
    }

    return seed;
  }

  private static void printByteArray(byte[] ba) {
    for (int i = 0; i < ba.length; i++) {
      System.out.print((int) ba[i]);
      if (i < ba.length-1) {
	System.out.print(", ");
      } else {
	System.out.println("");
      }
    }
  }

  public static void main(String argv[]) {
    KeyboardGenerator keyGen = new KeyboardGenerator();
    
    byte[] bytes = keyGen.seed();
    printByteArray(bytes);
  }
} // End of class KeyboardGenerator




